/*******************************************************************************
* YORKTOWN ADVANCED SGML PARSER (YASP)                                         *
*         An SGML Parser conforming to International Standard ISO 8879 --      *
*         Standard Generalized Markup Language                                 *
*------------------------------------------------------------------------------*
* Written 1987 - 1993 by                                                       *
*         Pierre G. Richard                                                    *
*         IBM T.J. Watson Research Center                                      *
*         Yorktown Heights, New York                                           *
*         petrus@watson.ibm.com                                                *
*------------------------------------------------------------------------------*
* FUNCTION NAME    = YASP / Header File                                        *
* STATUS           = Version 0, Release 1, Level 36                            *
* COMPONENT NAME   = banner.h                                                  *
* DESCRIPTIVE NAME = YASP Release, Version and Level numbers                   *
* DATE_WR          = 01/28/1988                                                *
* AUTHOR           = Pierre G. Richard - PETRUS at YKTVMX                      *
*                                                                              *
* FUNCTIONAL DESCRIPTION =                                                     *
*    Describe the YASP Release, Version and Level numbers as well as the       *
*    OCS and ODT version. These data serve two purposes:                       *
*    - generate appropriate banners to identify YASP and check the ODT and     *
*      OCS have the right version.                                             *
*    - permit conditional compilation when multiple version are being build.   *
*                                                                              *
* IMPLEMENTATION NOTES =                                                       *
*    GENBNR creates BANNER.C that is in turn compiled to provide external      *
*    references to the banners.   GENBNR uses the here #defined value          *
*    to creates the proper banners.                                            *
*                                                                              *
*    GENBNR is automatically called when a change occurs in this file.         *
*                                                                              *
* CHANGE HISTORY =                                                             *
* Number Date     Inits    Change & Reason (brief)                             *
*      1 03/30/91 PETRUS   Reworked to allow conditional compilation           *
*******************************************************************************/

#define YASP_FSV     "FSV0993MB127"
#define YASP_RELEASE  0
#define YASP_VERSION  1
#define YASP_LEVEL   36
#define YASP_OCS_VER 40
#define YASP_ODT_VER 40

#define YASP_RVL_EQ(R, V, L) \
   ((YASP_RELEASE == R)&&(YASP_VERSION == V)&&(YASP_LEVEL==L))

#define YASP_RVL_GE(R, V, L) \
   (YASP_RELEASE > R)|| \
   ((YASP_RELEASE >= R)&&((YASP_VERSION > V)||\
   ((YASP_VERSION >=V)&&(YASP_LEVEL >= L))))

/******************************************************************************/
